/*
    TCFbEColumnCachedTable.cpp    November 29, 2004.

    Copyright (C) 2003-2005 CFbE Research Group,
    Software Engineering Laboratory,
    Graduate School of Information Science,
    Nara Institute of Science and Technology,
    All rights reserved.

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, (at your option) or
    any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GNU Emacs; see the file COPYING.  If not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
//---------------------------------------------------------------------------
#pragma hdrstop

#include "TCFbEColumnCachedTable.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// コンストラクタ（空キャッシュを作成
__fastcall TCFbEColumnCachedTable::TCFbEColumnCachedTable(void)
    : TCFbECachedTable()
{
}

//---------------------------------------------------------------------------
// ParentTable の子テーブルを生成する
__fastcall TCFbEColumnCachedTable::TCFbEColumnCachedTable(TCFbECachedTable* ParentTable)
    : TCFbECachedTable(ParentTable)
{

}

//---------------------------------------------------------------------------
// デストラクタ
__fastcall TCFbEColumnCachedTable::~TCFbEColumnCachedTable()
{
}

//---------------------------------------------------------------------------
// 以下，protected method
//---------------------------------------------------------------------------
// 親キャッシュのセルが更新された
void __fastcall TCFbEColumnCachedTable::ParentCellIsUpdated(int RowIndex, int ColumnIndex)
{
    TCFbECachedTable::ParentCellIsUpdated(0, ColumnIndex);
}

//---------------------------------------------------------------------------
// 親キャッシュの行が追加された
int __fastcall  TCFbEColumnCachedTable::ParentAddRow(const char* RowLabel)
{
    return -1;  // 何もしない
}

//---------------------------------------------------------------------------
// 親キャッシュの列が追加された
int __fastcall  TCFbEColumnCachedTable::ParentAddColumn(const char* ColumnLabel)
{
    if (this->GetNumberOfRows() == 0) {
        this->AddRow("row_0");
    }
    return this->AddColumn(ColumnLabel);
}

//---------------------------------------------------------------------------
// 親キャッシュの行が削除された
void __fastcall TCFbEColumnCachedTable::ParentDeleteRowByIndex(int TargetRowIndex)
{
    return; // 何もしない
}

//---------------------------------------------------------------------------
// 親キャッシュの列が削除された
void __fastcall TCFbEColumnCachedTable::ParentDeleteColumnByIndex(int TargetColumnIndex)
{
    this->DeleteColumnByIndex(TargetColumnIndex);
}

//---------------------------------------------------------------------------
// 自分自身のクローンを作成する
TCFbECachedTable* __fastcall TCFbEColumnCachedTable::CreateClone(void)
{
    TCFbEColumnCachedTable* Clone = new TCFbEColumnCachedTable();
    Clone->Assign(this);
    return Clone;       
}

//---------------------------------------------------------------------------
// 自分自身の行と列が入れ替わったクローンを作成する
TCFbECachedTable* __fastcall TCFbEColumnCachedTable::CreateRotatedClone(void)
{
    TCFbERowCachedTable* Clone = new TCFbERowCachedTable();
    Clone->RotatedAssign(this);
    return Clone;
}

//---------------------------------------------------------------------------

